
# Import Modules
from calcalc.CalCalc import calculate
import math


# Is my output a float?
def test_output_type():
    isinstance(calculate("2**2"), float) == True


# Is my output numerically accurate?
def test_output_accurate():
    assert (4.0 - calculate("2**2")) < 0.0001


# Can Wolfram do math?
def test_wolfram():
    assert (4.0 - calculate("Two plus Two")) < 0.0001


# Can wolfram recall well-known, highly significant numerical values?
def test_wolf_num():
    assert calculate('What is the flight velocity of a sparrow?') == 25.0


# Can I pull string answers from Wolfram?
def test_wolf_string():
    assert calculate('Hello') == 'Hello, human.'


# Can the code handles nonsense?
def test_nonsense():
    assert math.isnan(calculate('rawr')) == True



#!/usr/bin/env python
import sys
from setuptools import setup

VERSION = '0.0.4'
DESCRIPTION = "AY250 HW3: evaluate string expressions with numexpr and Wolfram Alpha."

CLASSIFIERS = list(filter(None, map(str.strip,
"""
Development Status :: 2 - Pre-Alpha
Intended Audience :: Education
License :: OSI Approved :: BSD License
Programming Language :: Python
Programming Language :: Python :: 2.7
Programming Language :: Python :: 3.5
Programming Language :: Python :: 3.6
Programming Language :: Python :: 3.7
Programming Language :: Python :: 3.8
Programming Language :: Python :: 3.9
Topic :: Scientific/Engineering
""".splitlines())))

setup(
        name="CalcalcTT",
        version=VERSION,
        description=DESCRIPTION,
        long_description=DESCRIPTION,
        long_description_content_type="text/x-rst",
        classifiers=CLASSIFIERS,
        author="Toby Turney",
        author_email="tobyturney151@gmail.com",
        url="https://github.com/tobyturney151/python-ay250-homework/tobyturney_hw_3",
        python_requires='>=2.7',
        license="Livense to Kill Some Strings",
        packages=['calcalc', 'calcalc.tests'],
        platforms=['any'],
        setup_requires=['pytest-runner'],
        tests_require=['pytest'],
        install_requires=['numexpr', 'argparse']
)


# Note to self: hash-tags above a magic function are a no, no in Python.

# Import Modules
import urllib.request as request
import argparse
import numexpr as ne

# In order to get urllib.request to work from the command line,
# you HAVE to import it specifically, as above.
#
# When Calcalc.py is used from the command line,
# import urllib didn't actuallyimport everything, and I kept
# getting a AttributeError: module urllib has no attribute 'request.'


# Define a function (which will go into calculate) for accessing Wolfram Alpha.
def ask_wolfram(input_string):
    """
    Inputs
    ----------
    input_string: str
        A string to be passed to Wolfram Alpha.

    Output
    ----------
    answer_as_text: float
        The result from Wolfram Alpha, expressed as a float.
    """

    # put together query address
    url_prefix = "http://api.wolframalpha.com/v2/query?input="
    query = input_string
    query_nice = query.replace("+", "%2B").replace(
        " ", "+"
    )  # replace numeric '+' symbols; make '+' = space;
    query_opt = "&appid=6A8HR8-JR3LAA5QAT&format=plaintext&includepodid=Result"

    # decode utf-8 to text.
    full_url = url_prefix + query_nice + query_opt
    f = request.urlopen(full_url)
    f_as_text = f.read().decode("utf-8")
    # print(f_as_text)

    # pull out answer from plaintext section of query output
    idx_0 = f_as_text.find("<plaintext>") + 11
    idx_stop = f_as_text.find("</plaintext>", idx_0)
    # print(f_as_text[idx_0:idx_stop])

    # turn the first result into a number
    try:
        num = f_as_text[idx_0:idx_stop].split(" ")[0]
        answer = ne.evaluate(num.replace("×", "*").replace("^", "**")) + 0.0
        return answer

        # What if the result isn't a number at all?
    except (KeyError, TypeError):
        answer = f_as_text[idx_0:idx_stop]
        return answer


# Define function to evaluate an input string with numexpr.
def calculate(input_string):
    """
    Inputs
    ----------
    input_string: str
        A string to be passed to numexpr.evaluate.

    Output
    ----------
    answer: float
        The result of numexpr.evaluate(string), expressed as a float.
    """

    # Check that the string is actually a str.
    if type(input_string) != str:
        raise ValueError("The input isn't a string. Check input syntax")

    # Evaluate input_string with numexpr.evaluate
    try:
        answer = ne.evaluate(input_string) + 0.0
        answer.astype(float)
        return answer
    except (KeyError, SyntaxError):
        print(f'Could not evaluate \'{input_string}\'. Trying Wolfram Alpha')

        # Run wolfram alpha if numexpr can't solve it!
        try:
            answer = ask_wolfram(input_string)
            return answer
        except (ValueError, SyntaxError):
            print("Wolfram could not solve the query. Returning NaN.")
            answer = float("nan")
            return answer


# Parse command line arguments (when using from command line)
# Hash this out if you just want to test individual functions
# in the Jupyter Notebook.
if __name__ == "__main__":
    parser = argparse.ArgumentParser(description="Calcalc - calculate")
    parser.add_argument(
        "input_string",
        help="Input string to be evaluated. Required positional argument.",
    )
    answer = parser.parse_args()

    # Execute Function!
    print(calculate(answer.input_string))